﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter {
  using System.Collections.Generic;
  using System.Threading;
  using System.Threading.Tasks;
  using Core;
  using Expressions;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;

  using Stack;

  public interface IPushInterpreter {
    IRandom Random { get; set; }
    IPushStack<Expression> CodeStack { get; }
    IPushStack<Expression> ExecStack { get; }
    IPushStack<string> NameStack { get; }
    IPushStack<bool> BooleanStack { get; }
    IPushStack<long> IntegerStack { get; }
    IPushStack<double> FloatStack { get; }
    IPushStack<char> CharStack { get; }
    IPushStack<string> StringStack { get; }
    IPushStack<IReadOnlyList<long>> IntegerVectorStack { get; }
    IPushStack<IReadOnlyList<double>> FloatVectorStack { get; }
    IPushStack<IReadOnlyList<bool>> BooleanVectorStack { get; }
    IPushStack<IReadOnlyList<string>> StringVectorStack { get; }
    IPrintStack PrintStack { get; }
    IDictionary<string, Expression> CustomExpressions { get; }
    IReadOnlyDictionary<StackTypes, IPushStack> Stacks { get; }
    IReadOnlyPushConfiguration Configuration { get; }
    void Clear();
    void Reset();

    void SetInput(
      IReadOnlyList<long> integers = null,
      IReadOnlyList<double> floats = null,
      IReadOnlyList<bool> booleans = null,
      IReadOnlyList<char> chars = null,
      IReadOnlyList<string> strings = null,
      IReadOnlyList<IReadOnlyList<long>> integerVectors = null,
      IReadOnlyList<IReadOnlyList<double>> floatVectors = null,
      IReadOnlyList<IReadOnlyList<string>> stringVectors = null);

    void Run(bool stepwise);
    void Run(string code, bool stepwise = false);
    void Run(Expression expression, bool stepwise = false);
    Task RunAsync(Expression expression, CancellationToken token = default(CancellationToken));
    Task RunAsync(string code, CancellationToken token = default(CancellationToken));
    Task AbortAndResetAsync();
    Task AbortAsync();
    Task PauseAsync();
    void Resume();
    Task ResumeAsync();
    bool Step();
  }
}